#!/usr/bin/env python3
from argparse import ArgumentParser
from json import load
from os import path, mkdir
import sys

from PySide6 import QtWidgets
from PySide6.QtCore import Qt
from PySide6.QtWidgets import QApplication, QMainWindow, QFrame, QVBoxLayout, QLabel

from radispawn.radial_widget import RadialMenu, Wedge

class MainWindow(QMainWindow):
    def __init__(self, geom, wedges):
        QMainWindow.__init__(self)
        self.setAttribute(Qt.WA_TranslucentBackground)
        self.setGeometry(geom)

        self.frame = QFrame()
        self.vbox = QVBoxLayout()

        self.menu = RadialMenu(wedges)

        self.vbox.addWidget(self.menu, Qt.AlignCenter, Qt.AlignCenter)
        self.frame.setLayout(self.vbox)
        self.setCentralWidget(self.frame)

        self.showMaximized()

def getWedges(data):
    if "wedges" not in data.keys() or len(data["wedges"]) == 0:
        raise KeyError("json file must contain at least one wedge.")
    wedges = []
    for wedge in data["wedges"]:
        wedges.append(Wedge(wedge["name"], wedge["call"], *wedge["color"]))
    wedges = [wedges[0], *reversed(wedges[1:])]
    return wedges

def start(data):
    wedges = getWedges(data)

    app = QApplication(sys.argv)
    window = MainWindow(app.primaryScreen().availableGeometry(), wedges)
    sys.exit(app.exec())


def main():
    # Create config folder if it doesn't exist
    if not path.exists(pth := path.expanduser("~/.config/radispawn")):
        mkdir(pth)

    # Parse args with argparse
    parser = ArgumentParser()
    parser.add_argument("file", help="JSON file to define the menu; relative paths search pwd and then ~/.config/radispawn/")
    args = parser.parse_args()

    # fa stands for file argument
    fa = args.file
    json_file = None
    if path.exists(fa):
        json_file = fa
    # cf stands for .config file
    elif path.exists(cf := path.expanduser(f"~/.config/radispawn/{fa}")) and fa[0] != "/":
        json_file = cf
    else:
        raise FileNotFoundError(f"json file {fa} not found")
    
    with open(json_file, "r") as f:
        data = load(f)
    print(data)
    start(data)

if __name__ == "__main__":
    main()



import sys
from subprocess import Popen

from PySide6 import QtWidgets
from PySide6.QtWidgets import QApplication, QMainWindow, QWidget
from PySide6.QtGui import QPainter, QPen, QBrush, QColor
from PySide6.QtCore import QRect, Qt

class Wedge:
    def __init__(self, label, call, r, g, b):
        self.label = label
        self.color = QColor.fromRgb(r, g, b)
        self.call = call

class RadialMenu(QWidget):
    def __init__(self, wedges, outer_radius=200, inner_radius=100, select_color=0x000000, select_width=10):
        QWidget.__init__(self)

        self.outer_radius = outer_radius
        self.inner_radius = inner_radius
        self.select_color = select_color
        self.select_width = select_width
        self.wedges = wedges
        self.selected_wedge = 0

        # Needs to extend to the edge of the border around the outer radius
        self.resize((self.outer_radius + self.select_width) * 2, (self.outer_radius + self.select_width) * 2)
        self.setMinimumSize((self.outer_radius + self.select_width) * 2, (self.outer_radius + self.select_width) * 2)

        # Required for keyboard inputs
        self.setEnabled(True)
        self.grabKeyboard()

    # Select the next wedge clockwise
    def nextWedge(self):
        self.selected_wedge -= 1
        if self.selected_wedge < 0:
            self.selected_wedge += len(self.wedges)
        self.update()
    
    # Select the next wedge counterclockwise
    def prevWedge(self):
        self.selected_wedge += 1
        if self.selected_wedge == len(self.wedges):
            self.selected_wedge = 0
        self.update()

    def keyPressEvent(self, event):
        key = event.key()
        if key == Qt.Key_Right:
            self.nextWedge()
        elif key == Qt.Key_Left:
            self.prevWedge()
        elif key == Qt.Key_Return:
            Popen(self.wedges[self.selected_wedge].call.split())
            sys.exit(0)
        else:
            sys.exit(0)

    def paintEvent(self, event):
        # Angle of each wedge
        angle = 360 * 16 / len(self.wedges)
        # Angle of the right edge of the first wedge
        base_angle = 90 * 16 - angle / 2

        painter = QPainter()
        painter.begin(self)

        painter.setPen(Qt.NoPen)
        painter.setRenderHint(QPainter.Antialiasing)

        # Draw wedges as pie slices
        painter.setCompositionMode(QPainter.CompositionMode_SourceOver)
        for idx, wedge in enumerate(self.wedges):
            painter.setBrush(QBrush(wedge.color))
            # Extend from a distance from the edge that enables the selection border to be visible
            painter.drawPie(self.select_width, self.select_width, self.outer_radius * 2, self.outer_radius * 2, base_angle + angle * idx, angle)

        # Erase center of donut
        painter.setCompositionMode(QPainter.CompositionMode_DestinationOut)
        # Center is outer + sel_wid, so corners are that +/- inner
        painter.drawEllipse(self.outer_radius - self.inner_radius + self.select_width, self.outer_radius - self.inner_radius + self.select_width, self.inner_radius * 2, self.inner_radius * 2)

        # Draw border, same logic as the drawPie call above
        painter.setCompositionMode(QPainter.CompositionMode_SourceOver)
        pen = QPen(self.select_color)
        pen.setWidth(self.select_width)
        pen.setCapStyle(Qt.RoundCap)
        painter.setPen(pen)
        painter.drawArc(self.select_width, self.select_width, self.outer_radius * 2, self.outer_radius * 2, base_angle + angle * self.selected_wedge, angle)

        painter.end()
